from pathlib import Path
from tempfile import mkdtemp

from rich.console import Console
from rich.panel import Panel
from rich.syntax import Syntax

from thefuzz import fuzz

from .models import Config
from .questions import read_questions
from . import templates

import click
from importlib.metadata import version

CONFIG_PATH = "./faq.yml"
QUESTIONS_PATH = "./questions"
TEMPLATES_PATH = ".faq"
FAQ_PATH = "./FAQ.md"
FAQ_URL = "https://github.com/willmcgugan/faqtory/blob/main/FAQ.md"


QUESTIONS_README = """
# Questions

Your questions should go in this directory.

Question files should be named with the extension ".question.md".

"""

FAQ_TEMPLATE = """
# Frequently Asked Questions 

{%- for question in questions %}
- [{{ question.title }}](#{{ question.slug }})
{%- endfor %}

<hr>

{%- for question in questions %}

<a name="{{ question.slug }}"></a>
## {{ question.title }}

{{ question.body }}

{%- endfor %}

<hr>

Generated by [FAQtory](https://github.com/willmcgugan/faqtory)
"""

SUGGEST_TEMPLATE = """\
{%- if questions -%}
Thank you for your issue!
{% if questions|length == 1 %}
We found the following entry in the [FAQ]({{ faq_url }}) which you may find helpful:
{%- else %}
We found the following entries in the [FAQ]({{ faq_url }}) which you may find helpful:
{%- endif %}

{% for question in questions %}
- [{{ question.title }}]({{ faq_url }}#{{ question.slug }})
{%- endfor %}

Feel free to close this issue if you found an answer in the FAQ. Otherwise, please give us a little time to review.

{%- else -%}
Thank you for your issue. Give us a little time to review it.

PS. You might want to check the [FAQ]({{ faq_url }}) if you haven't done so already.
{%- endif %}

<hr>

Generated by [FAQtory](https://github.com/willmcgugan/faqtory)
"""


@click.group()
@click.version_option(version("faqtory"))
def run():
    pass


@run.command()
@click.option(
    "--config", help="Path to config file", default=CONFIG_PATH, metavar="PATH"
)
@click.option(
    "--questions", help="Path to questions", default=QUESTIONS_PATH, metavar="PATH"
)
@click.option(
    "--templates", help="Path to templates", default=TEMPLATES_PATH, metavar="PATH"
)
@click.option(
    "--output", help="Path to generated FAQ", default=FAQ_PATH, metavar="PATH"
)
@click.option("--faq-url", help="FAQ URL", default=FAQ_URL, metavar="PATH")
@click.option(
    "--overwrite/-no-overwrite",
    help="Overwrite files if they exist",
    default=False,
)
def init(
    config: str,
    questions: str,
    templates: str,
    output: str,
    faq_url: str,
    overwrite: bool,
) -> None:
    console = Console()
    error_console = Console(stderr=True)

    DEFAULT_CONFIG = f"""\
# FAQtory settings (you may edit this file)

questions_path: "{questions}" # Where questions should be stored
output_path: "{output}" # Where FAQ.md should be generated 
templates_path: "{templates}" # Path to templates
faq_url: "{faq_url}" # Replace this with the URL to your FAQ.md\
"""

    def write_path(path: Path, text: str) -> bool:
        try:
            with path.open("w" if overwrite else "x") as write_file:
                write_file.write(text)
        except FileExistsError:
            error_console.print(
                f"[red]⚠[/] File {str(path)!r} exists, use [b]--overwrite[/b] to update"
            )
            return False
        except Exception as error:
            error_console.print(f"[red]⚠[/] Unable to write {path}; {error}")
            return False

        console.print(f"[green]✔[/] Wrote {str(path)!r}")
        return True

    def make_directory(path: Path) -> bool:
        try:
            path.mkdir(parents=True, exist_ok=True)
        except Exception as error:
            error_console.print(f"unable to create {str(path)!r} directory; {error}")
            return False
        console.print(f"[green]✔[/] Directory {str(path)!r} created (or exists)")
        return True

    if write_path(Path(config), DEFAULT_CONFIG):
        console.print(
            Panel(
                Syntax(DEFAULT_CONFIG, "yaml", line_numbers=True),
                title=config,
            ),
        )

    make_directory(Path(questions))
    make_directory(Path(templates))

    readme_path = Path(questions) / "README.md"
    write_path(readme_path, QUESTIONS_README)
    write_path(Path(templates) / "FAQ.md", FAQ_TEMPLATE)
    write_path(Path(templates) / "suggest.md", SUGGEST_TEMPLATE)


@run.command()
@click.option(
    "--config",
    help="Path to config file or - for stdout",
    default=CONFIG_PATH,
    metavar="PATH",
)
@click.option(
    "--echo/-no-echo",
    help="Write generated file to terminal",
    default=False,
)
def build(config: str, echo: bool) -> None:
    """Build FAQ.md"""
    console = Console()
    config_data = Config.read(Path(config))
    questions = read_questions(config_data.questions_path)

    faq = templates.render_faq(config_data.templates_path, questions=questions)

    if echo:
        console.print(Syntax(faq, "markdown"))

    Path(config_data.output_path).write_text(faq)


@run.command()
@click.argument("query")
@click.option(
    "--config", help="Path to config file", default=CONFIG_PATH, metavar="PATH"
)
def suggest(query: str, config: str) -> None:
    """Suggest FAQ entries"""
    config_data = Config.read(Path(config))
    questions = read_questions(config_data.questions_path)

    scored_results = [(question.match(query), question) for question in questions]
    scored_results.sort(key=lambda result: result[0])
    print(scored_results)

    results = [question for ratio, question in scored_results if ratio > 50]

    suggest = templates.render_suggest(
        config_data.templates_path,
        questions=results,
        faq_url=config_data.faq_url,
    )
    print(suggest)



from pathlib import Path
from typing import List


from thefuzz import fuzz
import frontmatter
from yaml import load, Loader


from pydantic import BaseModel


class Question(BaseModel):
    title: str
    body: str
    alt_titles: List[str] = []

    @property
    def slug(self) -> str:
        return self.title.lower().replace(" ", "-").replace("?", "")

    @classmethod
    def read(cls, path: Path) -> "Question":
        """Read a question (Markdown with frontmatter)

        Args:
            path (str): Path to markdown.

        Returns:
            Question: A Question object
        """
        question_data = frontmatter.load(path)
        content = question_data.content
        metadata = question_data.metadata
        question = cls(
            title=metadata.get("title", ""),
            body=content,
            alt_titles=metadata.get("alt_titles", []),
        )
        return question

    @property
    def titles(self) -> list[str]:
        """Get all titles including alternatives"""
        return [self.title, *self.alt_titles]

    def match(self, query: str) -> int:
        """Match this question against a query"""
        return max(fuzz.partial_ratio(query, title) for title in self.titles)


class Config(BaseModel):
    questions_path: str
    output_path: str
    templates_path: str
    faq_url: str

    @classmethod
    def read(cls, path: Path) -> "Config":
        with open(path, "rb") as config_file:
            config_data = load(config_file, Loader=Loader)

        def get_str(key: str) -> str:
            return config_data.get(key, "")

        config = cls(
            questions_path=get_str("questions_path"),
            output_path=get_str("output_path"),
            templates_path=get_str("templates_path"),
            faq_url=get_str("faq_url"),
        )
        return config



from pathlib import Path
from jinja2 import Environment, FileSystemLoader, select_autoescape


def render_faq(templates_path: str, **template_args) -> str:
    """Render FAQ.md"""
    env = Environment(
        loader=FileSystemLoader(templates_path), autoescape=select_autoescape()
    )

    template = env.get_template("FAQ.md")
    result = template.render(**template_args)
    return result


def render_suggest(templates_path: str, **template_args) -> str:
    """Render suggest.md"""
    env = Environment(
        loader=FileSystemLoader(templates_path), autoescape=select_autoescape()
    )

    template = env.get_template("suggest.md")
    result = template.render(**template_args)
    return result



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['faqtory']

package_data = \
{'': ['*']}

install_requires = \
['click>=8.1.3,<9.0.0',
 'jinja2>=3.1.2,<4.0.0',
 'pydantic>=1.10.2,<2.0.0',
 'python-frontmatter>=1.0.0,<2.0.0',
 'pyyaml>=6.0,<7.0',
 'rich>=12.6.0,<13.0.0',
 'thefuzz[speedup]>=0.19.0,<0.20.0']

entry_points = \
{'console_scripts': ['faqtory = faqtory.cli:run']}

setup_kwargs = {
    'name': 'faqtory',
    'version': '0.3.1',
    'description': 'Auto FAQ builder',
    'long_description': '# FAQtory\n\nThis is a tool to auto-generate Frequently Asked Questions (FAQs) documents.\n\nFAQtory compiles a [FAQ.md](./FAQ.md) from individual `.question.md` documents. By default this will create a GitHub flavoured Markdown document, but you can edit a template to produce whatever format you like.\n\n## Getting started\n\nInstall `faqtory` from PyPI. I\'m going to assume you know how to do this bit.\n\nRun the following from the directory you wish to store the FAQ document. \n\n```bash\nfaqtory init\n```\n\nThis will create the following files and directories:\n\n- `faq.yml` A configuration file which you can edit.\n- `./.faq/` A directory which stores templates.\n- `./questions/` A directory containing question documents.\n\n## Adding questions\n\nTo add questions create a file with the extension `.question.md` in the questions directory (`./questions/` if you are using the defaults).\n\nQuestion documents are Markdown with front-matter. Here\'s an example:\n\n```yml\n---\ntitle: "What does FAQ stand for?"\n\n---\n\nFAQ stands for *Frequently Asked Questions*\n```\n\nThe filename is unimportant, but a `title` is mandatory. The body of the question can include any Markdown.\n\n## Building\n\nRun the following command to build the FAQ:\n\n```bash\nfaqtory build\n```\n\nWith the default settings this will generate an [FAQ.md](./FAQ.md) file.\n\n## Roadmap\n\nFAQtory is a work in progress, and a few hours work. The ultimate goal is to build a GitHub action that suggests answers to an issue from the FAQ.\n',
    'author': 'Will McGugan',
    'author_email': 'willmcgugan@gmail.com',
    'maintainer': 'None',
    'maintainer_email': 'None',
    'url': 'None',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from pathlib import Path

from .models import Question


def read_questions(path: str) -> list[Question]:
    questions: list[Question] = []
    for question_path in Path(path).glob("*.question.md"):
        question = Question.read(question_path)
        questions.append(question)
    return questions
